# -*- coding: utf-8 -*-
from django.contrib import admin
from django import forms
from django.db.models import TextField
from django.utils.translation import ugettext_lazy as _

from image_cropping import ImageCroppingMixin
from tinymce.widgets import AdminTinyMCE
from jmb.core.admin.options import ExtendibleModelAdmin

from acomea.fondi.models import (
    FundCategory, Fund, Diary, CategoryArea,
    CachedTweet, Promoter, Award
)
from jmb.core.forms.models import AdminModelForm


class DiaryAdminForm(AdminModelForm):
    class Meta:
        model = Diary

    def clean_tweet(self):
        data = self.cleaned_data['tweet']
        output = data
        start_url_position = data.find("http")
        if start_url_position > -1:
            end_url_position = data.find(" ", start_url_position)
            if end_url_position == -1:
                end_url_position = len(data)
            if data.find("https") > -1:
                lunghezza_http = 8
            else:
                lunghezza_http = 7
            output = output.replace(
                data[start_url_position:end_url_position],
                "<a href='%s' target='_blank'>%s</a>" %
                (
                    data[start_url_position:end_url_position],
                    data[start_url_position+lunghezza_http:end_url_position]
                )
            )
        url_pos = output.find("URL")
        if url_pos != -1:
            if not 'title' in self.cleaned_data:
                raise forms.ValidationError("Manca il campo Titolo")
        if len(output) > 140:
            raise forms.ValidationError("Dopo l'espanzione il tweet è di %s caratteri" % len(output))
        return data


class FundCategoryAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_status_icon', 'ordering', 'title', 'code',
        'slug', 'get_edit_icon', 'get_delete_icon',
    )
    list_display_links = ('title',)
    prepopulated_fields = {"slug": ("title",)}
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),

        (None, {
            "fields": (
                'status', 'code', 'title', 'slug', 'ordering',
                'categories_area', 'description'
            ),
        }),
    )
    save_on_top = True


class FundAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_status_icon', 'ordering', 'title', 'code', 'slug', 'fund_category',
        'get_edit_icon', 'get_delete_icon',
    )
    list_display_links = ('title',)
    prepopulated_fields = {"slug": ("title",)}
    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            "fields": (
                'status', 'code', 'fund_category', 'ordering',
                'title', 'slug', 'assogestioni_category',
                'detail_page_url',
                'currency', 'benchmark',
                'scope',
                'risk_profile',
                ('isin_a1_class', 'expense_a1_class'),
                ('isin_a2_class', 'expense_a2_class'),
                ('isin_q2_class', 'expense_q2_class'),
                ('isin_p1_class', 'expense_p1_class'),
                ('isin_p2_class', 'expense_p2_class'),
                ('isin_c1_class', 'expense_c1_class'),
                ('isin_c2_class', 'expense_c2_class'),
                ('isin_a5_class', 'expense_a5_class'),
                ('isin_p5_class', 'expense_p5_class'),
                'promoters',
                'copromoters',
                'awards',
                'short_description', 'long_description',
                'document_path', 'twitter_id',
            )
        }),
        (_("analysys_and_yields"), {
            'fields': (
                'analysis_header_text', 'analysis_footer_text',
                'yields_header_text', 'yields_footer_text',
            ),
            'classes': ('collapse',)
        }),
        (_("representation_analysis"), {
            'fields': (
                'classes_analysis_representation',
                'title_analysis_representation',
                'rating_analysis_representation',
                'countries_analysis_representation',
                'deadline_analysis_representation',
                'duration_analysis_representation',
                'sectors_analysis_representation',
                'currencies_analysis_representation',
                'areas_analysis_representation'
            ),
            'classes': ('collapse',)
        }),
    )
    filter_horizontal = ('promoters', 'copromoters', 'awards',)

    formfield_overrides = {
        TextField: {'widget': AdminTinyMCE}
    }


class DiaryAdmin(ImageCroppingMixin, ExtendibleModelAdmin):
    list_display = (
        'get_status_icon', 'ordering', 'title', 'slug',
        'description', 'get_fund_string', 'date',
        'get_edit_icon', 'get_delete_icon'
    )
    list_display_links = ('title',)
    search_fields = ['slug', 'title']
    prepopulated_fields = {"slug": ("title",)}
    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create', 'last_modifier',
                'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'status', 'funds', 'title', 'slug',
                'date', 'description', 'thumbnail', 'cropping', 'tweet', 'text', 'hide'
            ),
        }),
    )
    form = DiaryAdminForm
    filter_horizontal = ('funds',)


class PromoterAdmin(ExtendibleModelAdmin):
    list_display = ('name', 'get_image', 'description', 'get_edit_icon')
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create', 'last_modifier',
                'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {"fields": (
            'name', 'image', 'description', 'short_description',
        )}),
    )

    def get_image(self, obj):
        return "<img src='%s' alt='%s' />" % (
            obj.image.url,
            obj.name
        )
    get_image.short_description = _('image')
    get_image.allow_tags = True


class AwardAdmin(ExtendibleModelAdmin):
    list_display = ('title', 'date_awd', 'url', 'get_image', 'description', 'get_edit_icon')
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create', 'last_modifier',
                'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {"fields": (
            'title', 'date_awd', 'url', 'image', 'description'
        )}),
    )

    def get_image(self, obj):
        return "<img src='%s' alt='%s' />" % (
            obj.image.url,
            obj.title
        )
    get_image.short_description = _('image')
    get_image.allow_tags = True


admin.site.register(FundCategory, FundCategoryAdmin)
admin.site.register(Fund, FundAdmin)
admin.site.register(Promoter, PromoterAdmin)
admin.site.register(Award, AwardAdmin)
admin.site.register(Diary, DiaryAdmin)
admin.site.register(CategoryArea)
admin.site.register(CachedTweet)