# -*- coding: utf-8 -*-

import copy
import datetime

from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.decorators import api_view
from rest_framework.renderers import JSONRenderer

from acomea.soap.models import AcomeaClientCms
from acomea.fondi.utils.generics import get_graph_dataset_line, compare_values_for_y_axis


class JSONResponse(HttpResponse):
    """
    An HttpResponse that renders its content into JSON.
    """
    def __init__(self, data, **kwargs):
        content = JSONRenderer().render(data)
        kwargs['content_type'] = 'application/json'
        super(JSONResponse, self).__init__(content, **kwargs)


@csrf_exempt
@api_view(['GET'])
def get_grafico_fondo_data(request):
    fund_code = request.GET.get("fund_code", None)
    classe = request.GET.get("classe", None)
    data_dal = request.GET.get("data_dal", None)
    data_al = request.GET.get("data_al", None)
    data_dal = datetime.datetime.strptime(data_dal, "%d-%m-%Y") if data_dal else\
        datetime.datetime.now() - datetime.timedelta(18*365/12)
    data_al = datetime.datetime.strptime(data_al, "%d-%m-%Y") if data_al else datetime.datetime.now()
    acomea_client = AcomeaClientCms()
    raw_data = acomea_client.get_grafico(fund_code, classe, data_dal, data_al)
    dsl = copy.deepcopy(get_graph_dataset_line("container-line-%s" % classe))
    
    dsl['series'] = []
    serie_fondo = {
        'name': 'Valore Fondo',
        'data': [],
        'color': '#3E576F'
    }
    serie_benchmark = {
        'name': 'Valore Benchmark',
        'data': [],
        'color': '#AA4643'
    }

    ymin, ymax = 0, 0
    months_dict = {}

    # Se non sono presenti valori ritorno solo i dati generali del grafico
    if raw_data['SeriesData']:
        first_item = raw_data['SeriesData'][0]
        ymin = compare_values_for_y_axis(first_item['valf'], first_item['valb'])
        ymax = compare_values_for_y_axis(first_item['valf'], first_item['valb'], ctype="max")
        for l in raw_data['SeriesData']:
            datepy = datetime.date(int(l['data'][0:4]), int(l['data'][5:7]), int(l['data'][8:10]))
            serie_fondo['data'].append((datepy.strftime("%d/%m/%Y"), float(l['valf'])))
            serie_benchmark['data'].append((datepy.strftime("%d/%m/%Y"), float(l['valb'])))
            min_y_calculated_value = compare_values_for_y_axis(l['valf'], l['valb'])
            max_y_calculated_value = compare_values_for_y_axis(l['valf'], l['valb'], ctype="max")
            ymin = min_y_calculated_value if min_y_calculated_value < ymin else ymin
            ymax = max_y_calculated_value if max_y_calculated_value > ymax else ymax
            mm_yyyy = datepy.strftime('%m %Y')
            if mm_yyyy not in months_dict:
                months_dict[mm_yyyy] = datepy

    dsl['yAxis']['min'] = ymin - 0.15
    dsl['yAxis']['max'] = ymax + 0.15
    months_list = list(months_dict.values())
    months_list.sort()
    months_list_str = []
    for m in months_list:
        months_list_str.append(m.strftime("%b %y"))
    dsl['xAxis']['categories'] = months_list_str
    dsl['series'].append(serie_fondo)
    if raw_data['HasBenchmark']:
        dsl['series'].append(serie_benchmark)
    return JSONResponse(
        {
            "line": dsl,
            "data_dal": data_dal,
            "data_al": data_al,
            "DescrFon": raw_data["DescrFon"],
            "PerfF": round(raw_data["PerfF"], 2),
            "HasBenchmark": raw_data["HasBenchmark"],
            "BenchDescr": raw_data["BenchDescr"],
            "PerfB": round(raw_data["PerfB"], 2),
        },
        status=201
    )

