# coding: utf-8
import datetime
from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _


class SchedaProdottoForm(forms.Form):
    classe = forms.ChoiceField(
        label=_("classe"),
        choices=[],
    )
    data_dal = forms.DateField(
        label=_("Valore dei tuoi investimenti al"),
        initial=datetime.date(2011, 6, 1),
        widget=forms.DateInput(
            attrs={'placeholder': 'gg/mm/aaaa'},
            format='%d/%m/%Y'
        ),
        required=False
    )
    data_al = forms.DateField(
        label=_("al"),
        initial=datetime.date.today,
        widget=forms.DateInput(
            attrs={'placeholder': 'gg/mm/aaaa'},
            format='%d/%m/%Y'
        ),
        required=False
    )

    def __init__(self, *args, **kwargs):
        fund = kwargs.pop('fund')
        super(SchedaProdottoForm, self).__init__(*args, **kwargs)
        choices = [
            (classe.upper(), classe.upper()) for classe in settings.FONDI_FUND_CLASSI_LIST if getattr(
                fund, 'isin_{}_class'.format(classe)
            )
        ]
        self.fields['classe'].choices = choices

    def clean(self):
        data = self.cleaned_data
        if not data.get('data_dal'):
            data['data_dal'] = datetime.date(2011, 6, 1)
        if not data.get('data_al'):
            data['data_al'] = datetime.date.today()

        if data['data_al'] < data['data_dal']:
            raise forms.ValidationError(
                _("Date non corrette: la data di inizio deve essere minore della data di fine."))

        return data
