# -*- coding: utf-8 -*-
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

from cms.models.pluginmodel import CMSPlugin

from jmb.core.db.cmsmodels import StatusModel

from acomea.fondi.models import Fund, FundCategory


class CachedTweetSettings(CMSPlugin):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.FONDI_CMSPLUGINS_TWEET_TEMPLATES,
        #editable=len(settings.FONDI_CMSPLUGINS_TWEET_TEMPLATES) > 1,
        default="fondi/cms_plugins/fondi_tweet_list.html"
    )

    class Meta:
        app_label = "fondi"

    def __unicode__(self):
        return self.template_name


class DiarySettings(CMSPlugin):
    slug = models.CharField(
        _("Slug"),
        max_length=150,
        blank=True,
        help_text=_("(Optional)Diary slug(s) filter, separated by ','")
    )
    fund_slug = models.CharField(
        _("Fund slug"),
        max_length=150, blank=True,
        help_text=_("(Optional)Diary's fund slug(s) filter, separated by ','")
    )
    exclusion_slug = models.CharField(
        _("Exclusion slug"),
        max_length=150, blank=True,
        help_text=_("(Optional)excluding slug(s) filter, separated by ','")
    )
    exclusion_fund_slug = models.CharField(
        _("Exclusion fund slug"),
        max_length=150, blank=True,
        help_text=_("(Optional)excluding Diary's fund slug(s) filter, separated by ','")
    )
    objects_per_page = models.IntegerField(
        _("Items per page"),
        default=10
    )
    ordering = models.CharField(
        _("Ordering"),
        max_length=100,
        default='ordering, date'
    )
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.FONDI_CMSPLUGINS_DIARY_TEMPLATES,
        editable=len(settings.FONDI_CMSPLUGINS_DIARY_TEMPLATES) > 1,
        default="fondi/cms_plugins/fondi_diary_list.html"
    )

    class Meta:
        app_label = "fondi"

    def __unicode__(self):
        return self.template_name


class FundDiarySettings(CMSPlugin):
    title = models.CharField(
        _("Title"),
        max_length=150, blank=True,
        default="Diario del fondo"
    )
    fund = models.ForeignKey(
        Fund,
        related_name='diary_settings'
    )
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.FONDI_CMSPLUGINS_DIARY_SETTINGS_TEMPLATES,
        default="bootstrap3/fondi/cms_plugins/fund_diary.html"
    )
    viewable_diaries_number = models.IntegerField(
        _("Viewable diaries number"),
        default=5
    )
    show_title = models.BooleanField(
        _("Show title"),
        default=True,
    )

    class Meta:
        app_label = "fondi"

    def __unicode__(self):
        return "Diario %s (%s)" % (
            self.fund.title, self.fund.code,
        )


class FundSettings(CMSPlugin):
    category_slug = models.CharField(
        _("Category slug"),
        max_length=150, blank=True,
        help_text=_("(Optional)Fund Category slug(s) filter, separated by ','")
    )
    slug = models.CharField(
        _("Slug"),
        max_length=150, blank=True,
        help_text=_("(Optional)Fund slug(s) filter, separated by ','")
    )
    exclusion_slug = models.CharField(
        _("Exclusion slug"),
        max_length=150, blank=True,
        help_text=_("(Optional)excluding Fund slug(s) filter, separated by ','")
    )
    exclusion_category_slug = models.CharField(
        _("Exclusion category slug"),
        max_length=150, blank=True,
        help_text=_("(Optional)excluding Fund category slug(s) filter, separated by ','")
    )
    objects_per_page = models.IntegerField(
        _("Items per page"),
        default=10
    )
    ordering = models.CharField(
        _("Ordering"),
        max_length=150,
        default='ordering, date'
    )
    template_name = models.CharField(
        _("Template"),
        max_length=150,
        choices=settings.FONDI_CMSPLUGINS_FUND_TEMPLATES,
        default="cms_plugins/fondi/fondi_fund_list.html"
    )

    class Meta:
        app_label = "fondi"

    def __unicode__(self):
        if self.category_slug:
            return self.category_slug
        if self.slug:
            return self.slug
        return self.template_name


class FundCategorySettings(CMSPlugin):
    slug = models.CharField(
        _("Slug"),
        max_length=150, blank=True,
        help_text=_("(Optional)Category slug(s) filter, separated by ','")
    )
    exclusion_slug = models.CharField(
        _("Exclusion slug"),
        max_length=150, blank=True,
        help_text=_("(Optional)excluding Category slug(s) filter, separated by ','")
    )
    objects_per_page = models.IntegerField(
        _("Items per page"),
        default=10
    )
    ordering = models.CharField(
        _("Ordering"),
        max_length=150,
        default='ordering, title'
    )
    template_name = models.CharField(
        _("Template"),
        max_length=150,
        choices=settings.FONDI_CMSPLUGINS_FUNDCATEGORIES_TEMPLATES,
        default="cms_plugins/fondi/fondi_fundcategory_list.html"
    )

    class Meta:
        app_label = "fondi"

    def __unicode__(self):
        if self.slug:
            return self.slug
        return self.template_name


class AssetManagementSettings(CMSPlugin):
    template_name = models.CharField(
        _("Template"),
        max_length=150,
        choices=settings.FONDI_CMSPLUGINS_ASSETMANAGEMENT_TEMPLATES,
        default="fondi/cms_plugins/fondi_assetmanagement_list.html"
    )

    class Meta:
        app_label = "fondi"

    def __unicode__(self):
        return self.template_name


class ListaCategoriaSettings(CMSPlugin, StatusModel):
    tab_pane_slug = models.SlugField(
        max_length=255,
        blank=True, null=True,
        help_text="""
            serve per collegarlo ad una nav tab: getbootstrap.com/javascript/#tabs
        """
    )
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.FONDI_CMSPLUGIN_LISTACATEGORIA_TEMPLATES,
        default="bootstrap3/fondi/cms_plugins/lista_categoria_fondi.html"
    )
    fund_category = models.ForeignKey(
        FundCategory,
        related_name='lista_categoria_settings'
    )

    class Meta:
        app_label = "fondi"

    def __unicode__(self):
        return "%s %s %s" % (
            ('On - ' if self.status else 'Off - '),
            self.fund_category.code,
            self.template_name
        )


class SchedaTecnicaSettings(CMSPlugin, StatusModel):
    tab_pane_slug = models.SlugField(
        max_length=255,
        blank=True, null=True,
        help_text="""
            serve per collegarlo ad una nav tab: getbootstrap.com/javascript/#tabs
        """
    )
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.FONDI_CMSPLUGIN_SCHEDATECNICA_TEMPLATES,
        default="bootstrap3/fondi/cms_plugins/scheda_tecnica.html"
    )
    fund = models.ForeignKey(
        Fund,
        related_name='scheda_tecnica_settings'
    )

    class Meta:
        app_label = "fondi"

    def __unicode__(self):
        return "%s %s %s" % (
            ('On - ' if self.status else 'Off - '),
            self.fund.code,
            self.template_name
        )


class AnalisiPortafoglioSettings(CMSPlugin, StatusModel):
    tab_pane_slug = models.SlugField(
        max_length=255,
        blank=True, null=True,
        help_text="""
            serve per collegarlo ad una nav tab: getbootstrap.com/javascript/#tabs
        """
    )
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.FONDI_CMSPLUGIN_ANALISIPORTAFOGLIO_TEMPLATES,
        default="bootstrap3/fondi/cms_plugins/analisi_portafoglio.html"
    )
    fund = models.ForeignKey(
        Fund,
        related_name='analisi_portafoglio_settings'
    )

    class Meta:
        app_label = "fondi"

    def __unicode__(self):
        return "%s %s %s" % (
            ('On - ' if self.status else 'Off - '),
            self.fund.code,
            self.template_name
        )


class RendimentiQuoteSettings(CMSPlugin, StatusModel):
    tab_pane_slug = models.SlugField(
        max_length=255,
        blank=True, null=True,
        help_text="""
            serve per collegarlo ad una nav tab: getbootstrap.com/javascript/#tabs
        """
    )
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.FONDI_CMSPLUGIN_RENDIMENTIQUOTE_TEMPLATES,
        default="bootstrap3/fondi/cms_plugins/rendimenti_quote.html"
    )
    fund = models.ForeignKey(
        Fund,
        related_name='rendimenti_quote_settings'
    )

    class Meta:
        app_label = "fondi"

    def __unicode__(self):
        return "%s %s %s" % (
            ('On - ' if self.status else 'Off - '),
            self.fund.code,
            self.template_name
        )


class FundDetailSettings(CMSPlugin, StatusModel):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.FONDI_CMSPLUGIN_FUND_DETAIL_TEMPLATES,
        default="bootstrap3/fondi/cms_plugins/fund_detail.html"
    )
    fund = models.ForeignKey(
        Fund,
        related_name='detail_settings'
    )

    class Meta:
        app_label = "fondi"

    def __unicode__(self):
        return u"{} {} ({})".format(
            'On' if self.status else 'Off',
            self.fund.code, self.template_name
        )


class SchedaProdottoSettings(CMSPlugin, StatusModel):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.FONDI_CMSPLUGIN_SCHEDAPRODOTTO_TEMPLATES,
        default="bootstrap3/fondi/cms_plugins/fund_scheda_prodotto_form.html"
    )
    fund = models.ForeignKey(
        Fund,
        related_name='scheda_prodotto_settings'
    )

    class Meta:
        app_label = "fondi"

    def __unicode__(self):
        return "%s %s %s" % (
            ('On - ' if self.status else 'Off - '),
            self.fund.code,
            self.template_name
        )
