# -*- coding: utf-8 -*-

import datetime

from acomea.fondi.utils.scheda_prodotto import genera_scheda_prodotto_pdf
from django.conf import settings
from django.http import HttpResponse
from django.utils.translation import ugettext as _
from django.core.paginator import Paginator
from django.core.urlresolvers import reverse
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from acomea.fondi.models import Diary, Fund, FundCategory, CachedTweet
from acomea.fondi.cms.models import (
    ListaCategoriaSettings, SchedaTecnicaSettings,
    AssetManagementSettings, DiarySettings, FundSettings,
    FundCategorySettings, CachedTweetSettings, FundDiarySettings,
    AnalisiPortafoglioSettings, RendimentiQuoteSettings,
    FundDetailSettings, SchedaProdottoSettings
)
from acomea.fondi.utils.analisi_portafoglio import get_analisi_portafoglio_datasets
from acomea.fondi.utils.rendimenti_quote import rendimenti_quote_data, get_dataset_pie
from . import forms



###################################
## CMS PLUGINS                    #
###################################
from jmb.core.middleware.force_response import ForceResponse


class CachedTweetPlugin(CMSPluginBase):
    model = CachedTweetSettings
    name = _("Tweet list")
    module = _("Funds")

    def render(self, context, instance, placeholder):
        page = 1
        self.render_template = instance.template_name
        #ordering = instance.ordering.lower().replace(" ","").split(",")

        objects = CachedTweet.objects.filter(status=1)

        par_get = {}

        request = context['request']
        if request.GET or 'show-fondi' in request.COOKIES:
            for key in request.GET.keys():
                if request.GET[key] != "":
                    par_get[key] = request.GET[key]

            if 'page' in par_get:
                page = par_get['page']
            if 'show-fondi' in request.COOKIES:
                if 'show-fondo-ALL' in request.COOKIES and request.COOKIES['show-fondo-ALL'] == 'false':
                    lista_fondi = []
                    for cod in request.COOKIES['show-fondi'].split(','):
                        try:
                            f = Fund.objects.get( code=cod[2:4], fund_category__code=cod[0:2] )
                            lista_fondi.append(f)
                        except Fund.DoesNotExist:
                            pass
                    objects = objects.filter(fund__in=lista_fondi)
        #objects = objects.order_by(*ordering)

        #objects = objects[:instance.objects_per_page]
        #       paginator = Paginator(objects, instance.objects_per_page)
        #       objects = paginator(page)

        context.update({
            'objects': objects,
            'instance': instance,
            'placeholder': placeholder,
        })
        return context


class DiaryPlugin(CMSPluginBase):

    model = DiarySettings
    name = _("Diary list")
    module = _("Funds")
    admin_preview = False

    def render(self, context, instance, placeholder):
        page = 1
        self.render_template = instance.template_name
        ordering = instance.ordering.lower().replace(" ", "").split(",")
        slugs = fund_slugs = exclusion_slugs = exclusion_fund_slugs = []
        objects = Diary.objects.filter(status=1, funds__status=1)
        if instance.slug:
            slugs = instance.slug.lower().replace(" ", "").split(",")
            objects = objects.filter(slug__in=slugs)
        if instance.fund_slug:
            fund_slugs = instance.fund_slug.lower().replace(" ", "").split(",")
            objects = objects.filter(funds__slug__in=fund_slugs)
        if instance.exclusion_slug:
            exclusion_slugs = instance.exclusion_slug.lower().replace(" ", "").split(",")
            objects = objects.exclude(slug__in=exclusion_slugs)
        if instance.exclusion_fund_slug:
            exclusion_fund_slugs = instance.exclusion_fund_slug.lower().replace(" ", "").split(",")
            objects = objects.exclude(funds__slug__in=exclusion_fund_slugs)

        par_get = {}

        request = context['request']
        if request.GET or 'show-fondi' in request.COOKIES:
            for key in request.GET.keys():
                if request.GET[key] != "":
                    par_get[key] = request.GET[key]

            if 'page' in par_get:
                page = int(par_get['page'])
            if 'show-fondi' in request.COOKIES:
                if 'show-fondo-ALL' in request.COOKIES and request.COOKIES['show-fondo-ALL'] == 'false':
                    lista_fondi = []
                    for cod in request.COOKIES['show-fondi'].split(','):
                        try:
                            f = Fund.objects.get(code=cod[2:4], fund_category__code=cod[0:2])
                            lista_fondi.append(f)
                        except Fund.DoesNotExist:
                            pass
                    objects = objects.filter(funds__in=lista_fondi)
            if 'fondi' in request.GET and len(request.GET['fondi']) > 0:
                fund_codes = map(lambda x: x[2:4], request.GET['fondi'].split(','))
                fund_category_codes = map(lambda x: x[0:2], request.GET['fondi'].split(','))
                objects = objects.filter(
                    funds__code__in=fund_codes,
                    funds__fund_category__code__in=fund_category_codes
                )
            if 'last-three-months' in par_get:
                today = datetime.date.today()
                print 'in last 3 months'
                threemonthsago = today - datetime.timedelta(days=90)
                objects = objects.filter(date__gte=threemonthsago, date__lte=today)

        objects = objects.order_by('title').distinct('title')
        objects = sorted(objects, key=lambda x: x.date or datetime.date(1900, 1, 1), reverse=True)

        # Paginator
        paginator = Paginator(objects, instance.objects_per_page)
        adjacent_pages = 2
        page_numbers = [n for n in range(page-adjacent_pages, page+adjacent_pages+1) if 0 < n <= paginator.num_pages]
        show_first = True if 1 not in page_numbers else False
        show_last = True if paginator.num_pages not in page_numbers else False
        if page <= paginator.num_pages:
            objects = paginator.page(page)
        else:
            objects = paginator.page(paginator.num_pages)
            page = paginator.num_pages

        context.update({
            'page': page,
            'prev_page': page-1,
            'next_page': page+1,
            'show_first': show_first,
            'show_last': show_last,
            'page_numbers': page_numbers,
            'paginator': paginator,
            'user': request.user,
            'objects': objects,
            'instance': instance,
            'placeholder': placeholder,
        })
        return context


class FundDiaryPlugin(CMSPluginBase):

    model = FundDiarySettings
    name = _("Fund Diary plugin")
    module = _("Funds")

    def render(self, context, instance, placeholder):
        self.render_template = instance.template_name
        diaries = instance.fund.diaries.all()[:instance.viewable_diaries_number]
        context.update({
            'instance': instance,
            'diaries': diaries,
        })
        return context


class FundPlugin(CMSPluginBase):

    model = FundSettings
    name = _("Fund plugin")
    module = _("Funds")

    def render(self, context, instance, placeholder):
        self.render_template = instance.template_name
        objects = Fund.objects.filter(status=1)

        page = 1
        self.render_template = instance.template_name
        ordering = instance.ordering.lower().replace(" ", "").split(",")
        slugs = fund_slugs = exclusion_slugs = exclusion_fund_slugs = []

        if instance.slug:
            slugs = instance.slug.lower().replace(" ", "").split(",")
            objects = objects.filter(slug__in=slugs)
        if instance.category_slug:
            category_slugs = instance.category_slug.lower().replace(" ", "").split(",")
            objects = objects.filter(fund_category__slug__in=category_slugs)
        if instance.exclusion_slug:
            exclusion_slugs = instance.exclusion_slug.lower().replace(" ", "").split(",")
            objects = objects.exclude(slug__in=exclusion_slugs)
        if instance.exclusion_category_slug:
            exclusion_category_slugs = instance.exclusion_category_slug.lower().replace(" ", "").split(",")
            objects = objects.exclude(fund_category__slug__in=exclusion_category_slugs)

        context.update({
            'objects': objects,
            'instance': instance,
            'placeholder': placeholder,
        })
        return context


class FundCategoryList(CMSPluginBase):
    model = FundCategorySettings
    name = _("FundCategory list")
    module = _("Funds")

    def render(self, context, instance, placeholder):
        self.render_template = instance.template_name
        objects = FundCategory.objects.filter(status=1)

        page = 1
        self.render_template = instance.template_name
        ordering = instance.ordering.lower().replace(" ", "").split(",")
        slugs = exclusion_slugs = []

        if instance.slug:
            slugs = instance.slug.lower().replace(" ", "").split(",")
            objects = objects.filter(slug__in=slugs)
        if instance.exclusion_slug:
            exclusion_slugs = instance.exclusion_slug.lower().replace(" ", "").split(",")
            objects = objects.exclude(slug__in=exclusion_slugs)

        context.update({
            'objects': objects,
            'instance': instance,
            'placeholder': placeholder,
        })
        return context


class AssetManagementListPlugin(CMSPluginBase):
    model = AssetManagementSettings
    name = _("Asset management list")
    module = _("Funds")

    def render(self, context, instance, placeholder):
        self.render_template = instance.template_name
        request = context['request']

        child_list = []
        for child in request.current_page.get_children():
            if child.get_template_display() == 'MODAL' and child.published:
                child_list.append(child)

        context.update({
            'page_list': child_list,
            'instance': instance,
            'placeholder': placeholder,
        })
        return context


class ListaCategoriaPlugin(CMSPluginBase):
    model = ListaCategoriaSettings
    name = _("Lista Categoria Plugin")
    module = _("Fondi - Bootstrap3")
    admin_preview = False

    def render(self, context, instance, placeholder):
        self.render_template = "bootstrap3/empty.html"

        if not instance.status:
            return context

        if instance.fund_category.status == 0:
            return context

        self.render_template = instance.template_name
        funds = instance.fund_category.funds.filter(status=1).order_by('ordering')

        context.update({
            'object': instance,
            'placeholder': placeholder,
            'category': instance.fund_category,
            'funds': funds
        })
        return context


class SchedaTecnicaPlugin(CMSPluginBase):
    model = SchedaTecnicaSettings
    name = _("Scheda Tecnica Plugin")
    module = _("Fondi - Bootstrap3")
    admin_preview = False
    render_template = "bootstrap3/empty.html"

    def render(self, context, instance, placeholder):
        if not instance.status:
            return context

        if instance.fund.status == 0:
            return context

        if instance.fund.fund_category.status == 0:
            return context

        self.render_template = instance.template_name
        context.update({
            'object': instance,
            'placeholder': placeholder,
            'fund': instance.fund
        })
        return context


class AnalisiPortafoglioPlugin(CMSPluginBase):
    model = AnalisiPortafoglioSettings
    name = _("Analisi Portafoglio Plugin")
    module = _("Fondi - Bootstrap3")
    admin_preview = False
    render_template = "bootstrap3/empty.html"

    def render(self, context, instance, placeholder):
        if not instance.status:
            return context

        if instance.fund.status == 0:
            return context

        if instance.fund.fund_category.status == 0:
            return context

        self.render_template = instance.template_name
        context.update(get_analisi_portafoglio_datasets(instance.fund))
        context.update({
            'object': instance,
            'placeholder': placeholder,
            'fund': instance.fund,
        })
        return context


class RendimentiQuotePlugin(CMSPluginBase):
    model = RendimentiQuoteSettings
    name = _("Rendimenti e Quote Plugin")
    module = _("Fondi - Bootstrap3")
    admin_preview = False
    render_template = "bootstrap3/empty.html"

    def render(self, context, instance, placeholder):
        request = context["request"]

        if not instance.status:
            return context

        if instance.fund.status == 0:
            return context

        if instance.fund.fund_category.status == 0:
            return context

        self.render_template = instance.template_name
        data_riferimento = None if "data_rif" not in request.GET else \
            datetime.datetime.strptime(request.GET['data_rif'], "%d-%m-%Y")
        context.update({
            'object': instance,
            'placeholder': placeholder,
            'fund': instance.fund,
            'classi': ('A1', 'A2', 'Q2', 'C1', 'C2', 'A5', 'P5'),
            'data': rendimenti_quote_data(instance.fund, data_riferimento),
            'data_set_pie': get_dataset_pie(),
        })
        return context


class FundDetailPlugin(CMSPluginBase):
    model = FundDetailSettings
    name = _("Fund detail Plugin")
    module = _("Fondi - Bootstrap3")
    admin_preview = False
    render_template = "bootstrap3/empty.html"

    def render(self, context, instance, placeholder):
        if not instance.status:
            return context
        if instance.fund.status == 0:
            return context
        if instance.fund.fund_category.status == 0:
            return context
        self.render_template = instance.template_name
        url_kwargs = {'fund_id': instance.fund.id}
        context.update({
            'object': instance,
            'placeholder': placeholder,
            'fund': instance.fund,
            'url_scheda_tecnica':  reverse('fondi:scheda_tecnica', kwargs=url_kwargs),
            'url_analisi_portafoglio':  reverse('fondi:analisi_portafoglio', kwargs=url_kwargs),
            'url_rendimenti_quote':  reverse('fondi:rendimenti_quote', kwargs=url_kwargs),
        })
        return context


class SchedaProdottoPlugin(CMSPluginBase):
    model = SchedaProdottoSettings
    name = _("Fund scheda prodotto Plugin")
    module = _("Fondi - Bootstrap3")
    admin_preview = False
    render_template = "bootstrap3/empty.html"
    # form = SchedaProdottoForm

    def render(self, context, instance, placeholder):
        if not instance.status:
            return context
        if instance.fund.status == 0:
            return context
        if instance.fund.fund_category.status == 0:
            return context

        request = context['request']
        if request.method == 'POST':
            form = forms.SchedaProdottoForm(fund=instance.fund, data=request.POST)
            if form.is_valid():
                fund = instance.fund
                classe = form.cleaned_data['classe']
                data_dal = form.cleaned_data['data_dal']
                data_al = form.cleaned_data['data_al']
                pdf = genera_scheda_prodotto_pdf(fund, classe, data_dal=data_dal, data_al=data_al)
                response = HttpResponse(pdf, content_type='application/pdf')
                response['Content-Disposition'] = 'attachment; filename="scheda_prodotto_{}_{}.pdf"'.format(
                    fund.slug, classe)
                raise ForceResponse(response)
        else:
            form = forms.SchedaProdottoForm(fund=instance.fund)

        self.render_template = instance.template_name
        context.update({
            'object': instance,
            'placeholder': placeholder,
            # 'fund': instance.fund,
            'form': form,
        })
        return context




plugin_pool.register_plugin(DiaryPlugin)
plugin_pool.register_plugin(CachedTweetPlugin)
plugin_pool.register_plugin(FundPlugin)
plugin_pool.register_plugin(FundDiaryPlugin)
plugin_pool.register_plugin(FundCategoryList)
plugin_pool.register_plugin(AssetManagementListPlugin)
plugin_pool.register_plugin(ListaCategoriaPlugin)
plugin_pool.register_plugin(SchedaTecnicaPlugin)
plugin_pool.register_plugin(AnalisiPortafoglioPlugin)
plugin_pool.register_plugin(RendimentiQuotePlugin)
plugin_pool.register_plugin(FundDetailPlugin)
plugin_pool.register_plugin(SchedaProdottoPlugin)
