# -*- coding: utf-8 -*-

import json
from bson import json_util
from acomea.soap.models import AcomeaClientCms

from .generics import get_graph_dataset_bar, get_graph_dataset_pie


def get_analisi_portafoglio_datasets(fund):
    fund_code = fund.code
    acomea_client = AcomeaClientCms()
    datasets = dict()
    datasets['dataset_classi_attivita'] = get_analisi_portafoglio_dataset(
        fund, acomea_client.get_classi_di_attivita(fund_code), 'classi-attivita-',
    )
    datasets['dataset_titoli'] = get_analisi_portafoglio_dataset(
        fund, acomea_client.get_titoli(fund_code), 'titoli-',
    )
    datasets['dataset_scadenza'] = get_analisi_portafoglio_dataset(
        fund, acomea_client.get_scadenza(fund_code), 'scadenza-',
    )
    datasets['dataset_rating'] = get_analisi_portafoglio_dataset(
        fund, acomea_client.get_rating(fund_code), 'rating-',
    )
    datasets['dataset_aree_geografiche'] = get_analisi_portafoglio_dataset(
        fund, acomea_client.get_aree_geografiche(fund_code), 'aree-geografiche-',
    )
    datasets['dataset_principali_paesi'] = get_analisi_portafoglio_dataset(
        fund, acomea_client.get_principali_paesi(fund_code), 'principali-paesi-',
    )
    datasets['dataset_settori'] = get_analisi_portafoglio_dataset(
        fund, acomea_client.get_settori(fund_code), 'settori-',
    )
    datasets['dataset_valute'] = get_analisi_portafoglio_dataset(
        fund, acomea_client.get_valute(fund_code), 'valute-',
    )
    return datasets


def get_analisi_portafoglio_dataset(fund, data, prefix_render_to):
    data_set = None
    dsb = get_graph_dataset_bar(prefix_render_to, show_in_legend=False)
    if dsb and data:
        data_set = {}
        dsb['title']['text'] = data['line']['Titolo']
        dsb['subtitle']['text'] = data['line']['SottoTitolo']
        dsb['xAxis']['categories'] = data['line']['Categories']
        dsb['series'][0]['data'] = data['line']['SeriesData']
        dsb['plotOptions']['series']['dataLabels']['format'] = '{y:.2f}'
        # dst = data['line']['SeriesData']
        data_set['bar'] = json.dumps(dsb, default=json_util.default)
        dsp = get_graph_dataset_pie(prefix_render_to)
        dsp['title']['text'] = data['pie']['Titolo']
        dsp['subtitle']['text'] = data['pie']['SottoTitolo']
        dsp['xAxis']['categories'] = data['pie']['Categories']
        dsp['series'][0]['data'] = data['pie']['SeriesData']
        dst = data['pie']['SeriesData']
        data_set['pie'] = json.dumps(dsp, default=json_util.default)
        data_set['fund'] = fund
        data_set['table'] = dst
        data_set['last_update'] = data['line']['DataAggiornamento']
        data_set['active_buttons'] = data['active_buttons']
        if 'HasAreeGeografiche' in data:
            data_set['aree_geografiche_ds'] = []
            for element in data['table']:
                if isinstance(element[0], str):
                    area_geografica_dict = dict(
                        descrizione=element[0].lstrip("---").strip(), valore=element[1], root=False
                    )
                    if not element[0].startswith("--- "):
                        area_geografica_dict['root'] = True
                    data_set['aree_geografiche_ds'].append(area_geografica_dict)
    return data_set
