# -*- coding: utf-8 -*-

import copy
import datetime
import json
from bson import json_util
from babel.numbers import format_currency

from acomea.fondi.utils.generics import get_graph_dataset_line, compare_values_for_y_axis
from acomea.soap.models import AcomeaClientCms


def rendimenti_quote_data(fund, data_riferimento):
    data = dict()
    for classe in ('A1', 'A2', 'Q2', 'C1', 'C2', 'A5', 'P5'):
        data[classe] = rendimenti_quote_classe_data(fund, data_riferimento, classe)
    return data


def rendimenti_quote_classe_data(fund, data_riferimento, classe):
    fund_code = fund.code
    acomea_client = AcomeaClientCms()

    andamento = acomea_client.get_andamento(fund_code, classe)
    andamento['NAVTOTALE'] = format_currency(andamento['NAVTOTALE'], '', locale='it_IT')

    data_dal = andamento['Data_Avvio']
    data_al = datetime.datetime.now()

    serie_storica = acomea_client.get_serie_storica_per_classe(fund_code, classe, data_riferimento)
    rendimenti_annui = acomea_client.get_rendimenti_annui(fund_code, classe)

    stacchi_cedola = []
    if classe == 'C1' or classe == 'C2':
        stacchi_cedola = acomea_client.get_stacchi_cedola(fund_code, classe)

    # Gestione grafico
    grafico_raw_data = acomea_client.get_grafico(fund_code, classe, data_dal, data_al)
    dsl = copy.deepcopy(get_graph_dataset_line("container-line-%s" % classe))
    dsl['series'] = []
    serie_fondo = {
        'name': 'Valore Fondo',
        'data': [],
        'color': '#3E576F'
    }
    serie_benchmark = {
        'name': 'Valore Benchmark',
        'data': [],
        'color': '#AA4643'
    }
    first_item = grafico_raw_data['SeriesData'][0]
    ymin = compare_values_for_y_axis(first_item['valf'], first_item['valb'])
    ymax = compare_values_for_y_axis(first_item['valf'], first_item['valb'], ctype="max")
    months_dict = {}
    for l in grafico_raw_data['SeriesData']:
        datepy = datetime.date(int(l['data'][0:4]), int(l['data'][5:7]), int(l['data'][8:10]))
        serie_fondo['data'].append((datepy.strftime("%d/%m/%Y"), float(l['valf'])))
        serie_benchmark['data'].append((datepy.strftime("%d/%m/%Y"), float(l['valb'])))
        min_y_calculated_value = compare_values_for_y_axis(l['valf'], l['valb'])
        max_y_calculated_value = compare_values_for_y_axis(l['valf'], l['valb'], ctype="max")
        ymin = min_y_calculated_value if min_y_calculated_value < ymin else ymin
        ymax = max_y_calculated_value if max_y_calculated_value > ymax else ymax
        mm_yyyy = datepy.strftime('%m %Y')
        if mm_yyyy not in months_dict:
            months_dict[mm_yyyy] = datepy
    dsl['yAxis']['min'] = ymin - 0.15
    dsl['yAxis']['max'] = ymax + 0.15
    months_list = list(months_dict.values())
    months_list.sort()
    months_list_str = []
    for m in months_list:
        months_list_str.append(m.strftime("%b %y"))
    dsl['xAxis']['categories'] = months_list_str
    dsl['series'].append(serie_fondo)
    if grafico_raw_data['HasBenchmark']:
        dsl['series'].append(serie_benchmark)
    data_set_grafico = {}
    data_set_grafico['data_dal'] = data_dal
    data_set_grafico['data_al'] = data_al
    data_set_grafico['classe'] = classe
    data_set_grafico['raw'] = grafico_raw_data
    data_set_grafico['line'] = json.dumps(dsl, default=json_util.default)
    return dict(
        andamento=andamento,
        serie_storica=serie_storica,
        rendimenti_annui=rendimenti_annui,
        data_set_grafico=data_set_grafico,
        stacchi_cedola=stacchi_cedola
    )






def get_dataset_pie():
    return dict(
        Firefox=45.0,
        IE=26.8,
        Safari=8.5,
        Opera=6.2,
        Others=0.7
    )