# -*- coding: utf-8 -*-
import cairosvg
import datetime
import math
import os
import pygal
import re
import shutil
import time
import zipfile

from tempfile import NamedTemporaryFile
from pygal.style import LightenStyle

from jmb.core.utils.appypod import AppyPodTemplate
from jmb.core.utils.functions import get_template_abspath

from acomea.soap.models import AcomeaClientCms

soap_client = AcomeaClientCms()


def genera_scheda_prodotto_pdf(fund, fund_class, data_dal, data_al, template_name=None):

    # preparo dizionari e nomi
    tmp_root_url = "/tmp"
    tmp_random_i = round(time.time() * 10000000)
    tmp_dir_url = os.path.join(tmp_root_url, "~{}_odt_contents".format(tmp_random_i))

    if not template_name:
        template_name = 'fondi/scheda_prodotto.odt'
    odt_original = zipfile.ZipFile(get_template_abspath(template_name))
    odt_fixed = os.path.join(tmp_root_url, "{}.odt".format(tmp_random_i))

    list_file_to_substitute = [
        os.path.join(tmp_dir_url, 'content.xml'),
        os.path.join(tmp_dir_url, 'META-INF/manifest.xml')
    ]
    # ATTENZIONE QUESTI NOMI DI FILE CAMBIANO OGNI VOLTA CHE SI SALVA L'ODT
    dict_to_substitute = {
        'Pictures/10000201000002A50000019BF8130831B09E2A68.png': {'name': 'Pictures/classi.png', 'content': None},
        'Pictures/10000201000002C0000001B0E737EEA9280A25FB.png': {'name': 'Pictures/paesi.png', 'content': None},
        'Pictures/10000201000002A50000019B24C6D6A753BA4D5F.png': {'name': 'Pictures/paesi.png', 'content': None},
        'Pictures/10000201000002A50000019BA0174959B04147BC.png': {'name': 'Pictures/rendimenti.png', 'content': None},
    }
    fund_class = fund_class.upper()

    # recupero info da soap
    # classi
    classi, classi_svg = genera_classi_attivita(fund)
    if classi_svg:
        dict_to_substitute['Pictures/10000201000002A50000019BF8130831B09E2A68.png']['content'] = classi_svg.name

    # paesi
    paesi, paesi_svg = genera_principali_paesi(fund)
    if paesi_svg:
        dict_to_substitute['Pictures/10000201000002C0000001B0E737EEA9280A25FB.png']['content'] = paesi_svg.name

    # controllo se esistono le aree le sostituisco a paesi
    # richiesto da cliente come forzatura per fondo globale: unico fondo con aree geografiche invece di principali paesi
    area, area_svg = genera_aree_geografiche(fund)
    if area_svg:
        dict_to_substitute['Pictures/10000201000002A50000019B24C6D6A753BA4D5F.png']['content'] = area_svg.name

    # rendimenti e quote
    rendimenti = genera_rendimenti(fund, fund_class, data_dal, data_al)
    if rendimenti:
        dict_to_substitute['Pictures/10000201000002A50000019BA0174959B04147BC.png']['content'] = rendimenti.get('file_svg').name

    rendimenti_annui = soap_client.get_rendimenti_annui(fund.code, fund_class)

    # estrazione e manipolazione odt
    odt_original.extractall(tmp_dir_url)

    for entry in list_file_to_substitute:
        with open(entry) as f:
            content = f.read()
        with open(entry, 'w') as f:
            for (key, value) in dict_to_substitute.iteritems():
                if value.get('content'):
                    content = re.sub(key, value.get('name'), content)
            f.write(content)

    with zipfile.ZipFile(odt_fixed, 'w') as out_zip:
        for zip_info in odt_original.infolist():
            entry = dict_to_substitute.get(zip_info.filename)
            if entry and entry.get('content'):
                out_zip.write(entry.get('content'), entry.get('name'))
                continue

            file_name = zip_info.filename
            file_url = os.path.join(tmp_dir_url, file_name)
            out_zip.write(file_url, file_name)

    # preparo dict per sostituzione odt
    context = {
        'fund': fund,
        'fund_class': fund_class,
        'fund_currency_display': fund.get_currency_display(),
        'fund_commissione_gestione': getattr(fund, 'expense_{}_class'.format(fund_class.lower())),
        'data_aggiornamento': '{0:%d/%m/%y}'.format(classi.get('line').get('DataAggiornamento')) if classi else '',
        'classi_title': classi['line']['Titolo'] if classi else 'Classi attività',
        'paesi_title': paesi['line']['Titolo'] if paesi_svg else None,
        'area_title': area['line']['Titolo'] if area_svg else None,
    }
    context.update({
        'rendimenti_annui': [
            (
                rend['anno'],
                "{0:.2f}".format(round(float(rend['valf']), 2))
            ) for rend in rendimenti_annui if int(rend['anno']) > 2010
        ]
    })
    context.update(rendimenti.get('rendimenti', {}))

    if fund_class=='C1' or fund_class=='C2':
        from dateutil import parser
        stacchi_cedola = soap_client.get_stacchi_cedola(fund.code, fund_class)
        for cedola in stacchi_cedola:
            data = parser.parse(cedola['data'])
            cedola['data']=data.strftime(format='%d-%m-%Y')
        context.update({'stacchi_cedola': stacchi_cedola})

    # genero pdf da odt
    pdf = AppyPodTemplate(odt_fixed, context).render(context=context)

    # clean file temp
    os.remove(odt_fixed)
    shutil.rmtree(tmp_dir_url)
    return pdf


def genera_classi_attivita(fund):
    return genera_horizontal_line_svg(soap_client.get_classi_di_attivita(fund.code))


def genera_principali_paesi(fund):
    return genera_horizontal_line_svg(soap_client.get_principali_paesi(fund.code))


def genera_aree_geografiche(fund):
    return genera_horizontal_line_svg(soap_client.get_aree_geografiche(fund.code))


def genera_horizontal_line_svg(dataset):
    if not dataset or not dataset.get('line'):
        return {}, None
    file_svg = NamedTemporaryFile('wb', delete=False)

    series = dataset.get('line').get('SeriesData')
    labels = dataset.get('line').get('Categories')[:len(series)]

    # Proporziono le labels in base ai caratteri con un max di 20 e un min di 10
    max_char = len(max(labels, key=len))
    label_font_size = 20 * 10 / max_char
    label_font_size = label_font_size if label_font_size > 8 else 10
    label_font_size = label_font_size if label_font_size < 20 else 20
    # # Proporziono i margini in base ai caratteri con un max di 20 e un min di 10
    # margin = -75 if max_char > 18 else 0
    # margin = -25 if max_char > 40 else margin
    series.reverse()
    labels.reverse()

    acomea_style = LightenStyle('#E65112', step=5, max_=10, label_font_size=label_font_size)
    bar_chart = pygal.HorizontalBar(
        # show_legend=False, fill=True, style=acomea_style, margin_left=margin
        show_legend=False, fill=True, style=acomea_style
    )
    bar_chart.title = dataset['line']['SottoTitolo']
    bar_chart.x_labels = []

    bar_chart.add('line', series)
    bar_chart.x_labels = labels

    bar_chart.render_to_file(file_svg.name)
    # fix pygal export svg, force regenerate correct svg
    cairosvg.svg2svg(url=file_svg.name, write_to=file_svg.name)

    return dataset, file_svg


def genera_rendimenti(fund, fund_class, data_dal, data_al):
    andamento = soap_client.get_andamento(fund.code, fund_class)
    try:
        data_avvio = andamento['Data_Avvio'].date()
        if data_dal < data_avvio:
            data_dal = data_avvio
    except:
        pass

    dataset = soap_client.get_grafico(fund.code, fund_class, data_dal=data_dal, data_al=data_al)

    if not dataset or not dataset.get('SeriesData'):
        return {}
    file_svg = NamedTemporaryFile('wb', delete=False)

    title_f = 'Valore Fondo'
    title_b = 'Valore Benchmark'

    values = dataset['SeriesData']

    labels_f = []
    values_f = []
    values_b = []
    values_f_are_null = True
    values_b_are_null = True
    ind = 0
    max_values = None

    if max_values:
        values = values[:max_values]

    for value in values:
        if int(len(values) / 10) > 0 and ind % int(len(values) / 10) != 0:
            label = ''
        else:
            datepy = datetime.date(int(value['data'][0:4]), int(value['data'][5:7]), int(value['data'][8:10]))
            label = datepy.strftime("%d/%m/%Y")
        labels_f.append(label)
        values_f.append(value.get('valf'))
        values_f_are_null = True if value.get('valf') == 0 and values_f_are_null else False
        values_b.append(value.get('valb'))
        values_b_are_null = True if value.get('valb') == 0 and values_b_are_null else False
        ind += 1

    range_graph = None

    if not values_f_are_null or not values_b_are_null:
        precision = 0.5
        if not values_f_are_null and not values_b_are_null:
            min_value = min(values_f + values_b)
            max_value = max(values_f + values_b)
        else:
            values = values_f if values_b_are_null else values_b
            min_value = min(values)
            max_value = max(values)
        min_value = precision * math.floor(2.0 * min_value)
        max_value = precision * math.ceil(2.0 * max_value)
        range_graph = (min_value, max_value)

    line_chart = pygal.Line(
        show_legend=True, legend_at_bottom=True, fill=False,
        human_readable=True, x_label_rotation=45, show_dots=False,
        range=range_graph
    )

    line_chart.x_labels = labels_f
    line_chart.add(title_f, values_f)
    line_chart.add(title_b, values_b)

    line_chart.render_to_file(file_svg.name)
    # fix pygal export svg, force regenerate correct svg
    cairosvg.svg2svg(url=file_svg.name, write_to=file_svg.name)

    rendimenti = {
        'dataset': dataset,
        'file_svg': file_svg,
        'rendimenti': {
            'rendimenti_data_dal': '{0:%d/%m/%y}'.format(data_dal),
            'rendimenti_data_al': '{0:%d/%m/%y}'.format(data_al),
            'rendimenti_fondo_title': dataset['DescrFon'],
            'rendimenti_fondo_value': "{0:.2f}".format(round(float(dataset['PerfF']), 2)),
            'rendimenti_benchmark_title': dataset['BenchDescr'],
            'rendimenti_benchmark_value': "{0:.2f}".format(round(float(dataset['PerfB']), 2)),
        },
    }

    return rendimenti


def test_genera_scheda_prodotto_pdf():
    from acomea.fondi import models
    pdf = open('/tmp/test.pdf', 'wb')
    fund = models.Fund.objects.get(pk=13)
    pdf.write(genera_scheda_prodotto_pdf(fund, 'A1', datetime.datetime(2011, 10, 01), datetime.date.today()))

    pdf.close()
