# -*- coding: utf-8 -*-

import datetime
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.views.generic import TemplateView
from django.views.decorators.csrf import csrf_exempt
from django.shortcuts import get_object_or_404

from .models import Fund
from .utils.analisi_portafoglio import get_analisi_portafoglio_datasets
from .utils.rendimenti_quote import rendimenti_quote_classe_data, get_dataset_pie


class SchedaTecnicaTemplateView(TemplateView):
    template_name = "bootstrap3/fondi/cms_plugins/fund_detail/scheda_tecnica.html"

    def get_context_data(self, **kwargs):
        fund_id = int(kwargs.get('fund_id'))
        context = super(SchedaTecnicaTemplateView, self).get_context_data(**kwargs)
        context['fund'] = get_object_or_404(Fund, pk=fund_id)
        return context


class AnalisiPortafoglioTemplateView(TemplateView):
    template_name = "bootstrap3/fondi/cms_plugins/fund_detail/analisi_portafoglio.html"

    def get_context_data(self, **kwargs):
        fund_id = int(kwargs.get('fund_id'))
        fund = get_object_or_404(Fund, pk=fund_id)
        context = super(AnalisiPortafoglioTemplateView, self).get_context_data(**kwargs)
        context['fund'] = fund
        context.update(get_analisi_portafoglio_datasets(fund))
        try:
            if '5gimme5' in self.request.META['HTTP_ORIGIN']:
                context['is_gimme5'] = True
        except:
            pass
        return context


class RendimentiQuoteTemplateView(TemplateView):
    template_name = "bootstrap3/fondi/cms_plugins/fund_detail/rendimenti_quote.html"

    def dispatch(self, request, *args, **kwargs):
        try:
            if '5gimme5' in request.META['HTTP_ORIGIN']:
                self.template_name = "gimme5/fondi/includes/_rendimenti_quote.html"
        except:
            pass
        return super(RendimentiQuoteTemplateView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        fund_id = int(kwargs.get('fund_id'))
        fund = get_object_or_404(Fund, pk=fund_id)
        context = super(RendimentiQuoteTemplateView, self).get_context_data(**kwargs)
        context.update({
            'fund': fund,
            'url_a1': reverse('fondi:rendimenti_quote_classe', kwargs={'fund_id': fund_id, 'classe': 'A1'}),
            'url_a2': reverse('fondi:rendimenti_quote_classe', kwargs={'fund_id': fund_id, 'classe': 'A2'}),
            'url_q2': reverse('fondi:rendimenti_quote_classe', kwargs={'fund_id': fund_id, 'classe': 'Q2'}),
            'url_p1': reverse('fondi:rendimenti_quote_classe', kwargs={'fund_id': fund_id, 'classe': 'P1'}),
            'url_p2': reverse('fondi:rendimenti_quote_classe', kwargs={'fund_id': fund_id, 'classe': 'P2'}),
            'url_c1': reverse('fondi:rendimenti_quote_classe', kwargs={'fund_id': fund_id, 'classe': 'C1'}),
            'url_c2': reverse('fondi:rendimenti_quote_classe', kwargs={'fund_id': fund_id, 'classe': 'C2'}),
            'url_a5': reverse('fondi:rendimenti_quote_classe', kwargs={'fund_id': fund_id, 'classe': 'A5'}),
            'url_p5': reverse('fondi:rendimenti_quote_classe', kwargs={'fund_id': fund_id, 'classe': 'P5'}),
        })
        return context


class RendimentiQuoteClasseTemplateView(TemplateView):
    template_name = "bootstrap3/fondi/cms_plugins/fund_detail/rendimenti_quote_classe.html"

    def dispatch(self, request, *args, **kwargs):
        try:
            if '5gimme5' in request.META['HTTP_ORIGIN']:
                self.template_name = "gimme5/fondi/includes/rendimenti_quote_classe.html"
        except:
            pass
        return super(RendimentiQuoteClasseTemplateView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        fund_id = int(kwargs.get('fund_id'))

        classe = kwargs.get('classe')
        fund = get_object_or_404(Fund, pk=fund_id)

        context = super(RendimentiQuoteClasseTemplateView, self).get_context_data(**kwargs)

        data_riferimento = None if "data_rif" not in self.request.GET else \
            datetime.datetime.strptime(self.request.GET['data_rif'], "%d-%m-%Y")
        context.update({
            'fund': fund,
            'classe': classe,
            'data': rendimenti_quote_classe_data(fund, data_riferimento, classe),
            'data_set_pie': get_dataset_pie()
        })
        return context


@csrf_exempt
def xset_select_fondi(request):
    try:
        response = HttpResponse("OK")
        if request.POST['setall'] == 'True':
            response.set_cookie( 'show-fondo-ALL', 'true', expires=datetime.datetime(2014, 1, 1))
        else:
            response.set_cookie( 'show-fondo-ALL', 'false', expires=datetime.datetime(2014, 1, 1) )
            response.set_cookie( 'show-fondi', request.POST['listacodfondi'], expires=datetime.datetime(2014, 1, 1) )
    except Exception, exc:
        print str(exc)
        return HttpResponse("KO")
    return response


@csrf_exempt
def load_lista_fondi(request):
    try:
        response = HttpResponse("OK")
        response.set_cookie( 'show-fondo-ALL', 'false', expires=datetime.datetime(2014, 1, 1) )
        response.set_cookie( 'show-fondi', request.POST['listacodfondi'], expires=datetime.datetime(2014, 1, 1) )
    except Exception, exc:
        print str(exc)
        return HttpResponse("KO")
    return response
